@@ -617,7 +617,7 @@ registry be in use, and should be called after the call to
      (require 'gnorb-gnus)
      (unless (gnus-registry-install-p)
        (user-error "Gnorb tracking requires that the Gnus registry be installed."))
-     (add-hook 'org-capture-prepare-finalize-hook #''gnorb-registry-capture)
+     (add-hook 'org-capture-prepare-finalize-hook #'gnorb-registry-capture)
      (setq gnorb-tracking-enabled t))))
 
 ;;;###autoload
@@ -646,6 +646,9 @@ registry be in use, and should be called after the call to
        (define-key gnus-summary-mime-map "a" #'gnorb-gnus-article-org-attach)
        (define-key gnus-summary-mode-map (kbd "C-c t") #'gnorb-gnus-incoming-do-todo)
        (define-key gnus-summary-mode-map (kbd "C-c v") #'gnorb-gnus-view)
+       (define-key gnus-summary-mode-map (kbd "C-c C-t") #'gnorb-gnus-tag-message)
+       (define-key gnus-summary-limit-map (kbd "g") #'gnorb-gnus-insert-tagged-messages)
+       (define-key gnus-summary-limit-map (kbd "G") #'gnorb-gnus-insert-tracked-messages)
        (setq gnorb-gnus-capture-always-attach t)
        (push '("attach to org heading" . gnorb-gnus-mime-org-attach)
              gnus-mime-action-alist)
